/*
 *  $Id: mpi.h,v 1.11 1994/02/01 02:37:39 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/* user include file for MPI programs */

#ifndef _MPI_INCLUDE
#define _MPI_INCLUDE

#include "mpi_errno.h"
#include "dmpiatom.h"
#include "mpi_bc.h"
#include "mpir.h"
#include "mpi_ad.h"

extern MPI_Comm MPI_COMM_WORLD;
extern void MPI_MAX();
extern void MPI_MIN();
extern void MPI_SUM();
extern void MPI_PROD();
extern void MPI_LAND();
extern void MPI_BAND();
extern void MPI_LOR();
extern void MPI_BOR();
extern void MPI_LXOR();
extern void MPI_BXOR();
extern void MPI_MAXLOC();
extern void MPI_MINLOC();

#endif

#if defined(MPI_rs6000)


#define mpi_abort_            mpi_abort
#define mpi_finalize_         mpi_finalize
#define mpi_init_             mpi_init
    
#define mpi_address_          mpi_address
#define mpi_cancel_           mpi_cancel
#define mpi_comm_init_        mpi_comm_init
#define mpi_commreq_free_     mpi_commreq_free
#define mpi_irecv_            mpi_irecv
#define mpi_isend_            mpi_isend
#define mpi_probe_            mpi_probe
#define mpi_recv_             mpi_recv
#define mpi_send_             mpi_send
#define mpi_start_            mpi_start
#define mpi_test_             mpi_test
#define mpi_testany_          mpi_testany
#define mpi_type_extent_      mpi_type_extent
#define mpi_type_contiguous_  mpi_type_contiguous
#define mpi_type_vector_      mpi_type_vector
#define mpi_type_hvector_     mpi_type_hvector
#define mpi_type_indexed_     mpi_type_indexed
#define mpi_type_hindexed_    mpi_type_hindexed
#define mpi_type_struct_      mpi_type_struct
#define mpi_wait_             mpi_wait
#define mpi_waitall_          mpi_waitall

#define mpi_attr_delval_      mpi_attr_delval
#define mpi_attr_freekey_     mpi_attr_freekey
#define mpi_attr_getkey_      mpi_attr_getkey
#define mpi_attr_getval_      mpi_attr_getval
#define mpi_attr_putval_      mpi_attr_putval
#define mpi_attr_util_        mpi_attr_util
#define mpi_comm_dup_         mpi_comm_dup
#define mpi_comm_free_        mpi_comm_free
#define mpi_comm_group_       mpi_comm_group
#define mpi_comm_make_        mpi_comm_make
#define mpi_comm_rank_        mpi_comm_rank
#define mpi_comm_rankf_       mpi_comm_rankf
#define mpi_comm_size_        mpi_comm_size
#define mpi_comm_sizef_       mpi_comm_sizef
#define mpi_comm_split_       mpi_comm_split
#define mpi_comm_test_ic_     mpi_comm_test_ic
#define mpi_comm_util_        mpi_comm_util
#define mpi_context_util_     mpi_context_util
#define mpi_group_diff_       mpi_group_diff
#define mpi_group_excl_       mpi_group_excl
#define mpi_group_free_       mpi_group_free
#define mpi_group_incl_       mpi_group_incl
#define mpi_group_inter_      mpi_group_inter
#define mpi_group_rank_       mpi_group_rank
#define mpi_group_rexcl_      mpi_group_rexcl
#define mpi_group_rincl_      mpi_group_rincl
#define mpi_group_size_       mpi_group_size
#define mpi_group_tranks_     mpi_group_tranks
#define mpi_group_union_      mpi_group_union
#define mpi_group_util_       mpi_group_util
#define mpi_ic_finish_        mpi_ic_finish
#define mpi_ic_start_         mpi_ic_start

#define mpi_get_source_       mpi_get_source
#define mpi_get_tag_          mpi_get_tag
#define mpi_get_count_        mpi_get_count

#define mpi_allgather_        mpi_allgather
#define mpi_allgatherv_       mpi_allgatherv
#define mpi_allreduce_        mpi_allreduce
#define mpi_allreducef_       mpi_allreducef
#define mpi_alltoall_         mpi_alltoall
#define mpi_alltoallv_        mpi_alltoallv
#define mpi_barrier_          mpi_barrier
#define mpi_bcast_            mpi_bcast
#define mpi_bcastf_           mpi_bcastf
#define mpi_gather_           mpi_gather
#define mpi_gatherv_          mpi_gatherv
#define mpi_global_ops_       mpi_global_ops
#define mpi_red_scat_         mpi_red_scat
#define mpi_reduce_           mpi_reduce
#define mpi_reduce_scat_      mpi_reduce_scat
#define mpi_reducef_          mpi_reducef
#define mpi_scan_             mpi_scan
#define mpi_scatter_          mpi_scatter
#define mpi_scatterv_         mpi_scatterv
#define mpi_u_allreduce_      mpi_u_allreduce
#define mpi_u_allreducea_     mpi_u_allreducea
#define mpi_u_red_scat_       mpi_u_red_scat
#define mpi_u_red_scata_      mpi_u_red_scata
#define mpi_u_reduce_         mpi_u_reduce
#define mpi_u_reducea_        mpi_u_reducea
#define mpi_u_scan_           mpi_u_scan
#define mpi_u_scana_          mpi_u_scana



#define mpi_cart_coords_      mpi_cart_coords
#define mpi_cart_rank_        mpi_cart_rank
#define mpi_cart_shift_       mpi_cart_shift
#define mpi_cart_shiftf_      mpi_cart_shiftf
#define mpi_cart_sub_         mpi_cart_sub
#define mpi_get_cart_         mpi_get_cart
#define mpi_get_cartdim_      mpi_get_cartdim
#define mpi_get_graph_        mpi_get_graph
#define mpi_get_graphdims_    mpi_get_graphdims
#define mpi_graph_nbr_        mpi_graph_nbr
#define mpi_graph_nbr_cnt_    mpi_graph_nbr_cnt
#define mpi_make_cart_        mpi_make_cart
#define mpi_make_cartf_       mpi_make_cartf
#define mpi_make_dims_        mpi_make_dims
#define mpi_make_graph_       mpi_make_graph
#define mpi_map_cart_         mpi_map_cart
#define mpi_map_graph_        mpi_map_graph
#define mpi_topo_status_      mpi_topo_status

#endif
