/*
 *  $Id: u_reduce.c,v 1.13 1994/02/03 02:22:50 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: u_reduce.c,v 1.13 1994/02/03 02:22:50 lusk Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

   MPI_User_reduce - 

   Currently, there is no (very little) difference between user reduce
   and reduce.  We should possibly change it so that MPI defined ops cannot
   be used in user reduce and only MPI defined ops can be used in reduce

@*/
int MPI_User_reduce ( sendbuf, recvbuf, count, datatype, op, root, comm )
void             *sendbuf;
void             *recvbuf;
int               count;
MPI_Datatype      datatype;
void            (*op)();
int               root;
MPI_Comm          comm;
{
  MPI_Status status;
  int        size, rank;
  int        errno = MPI_SUCCESS;
  int        temp;

  /* Check for invalid arguments */ 
  if ( ( (comm            == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL) ) ||
       ( (root            <  0)           && (errno = MPI_ERR_ROOT) )      || 
       ( (comm->comm_type == MPIR_INTER)  && (errno = MPI_ERR_COMM_INTER) ) )
    return (errno); 
  
  /* Is root within the communicator? */
  MPI_Comm_size ( comm, &size );
  if ( root >= size )
    return (errno);

  /* Get my rank and switch communicators to the hidden collective */
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* Initialize the op */
  MPIR_Init_coll_op ( sendbuf, count, datatype, op, rank );

  /* If I'm the root, then I will receive and user reduce the messages */
  if (rank == root) {
    int              extent, i;
    void            *buffer;

    /* Copy my buffer to the recvbuf */
    MPI_Type_extent ( datatype, &extent );
    buffer = (void *)MALLOC(extent * count);
    if (sendbuf != recvbuf) 
      MPI_Sendrecv( sendbuf, count, datatype, root, MPIR_USER_REDUCE_TAG,
		    recvbuf, count, datatype, root, MPIR_USER_REDUCE_TAG, 
		    comm, &status);

    /* Receive messages from everywhere and combine buffers */
    for (i=0; i<root; i++) {
      MPI_Recv( buffer, count, datatype, i, MPIR_USER_REDUCE_TAG, comm, &status);
	  temp = count;
      (*op)(buffer, recvbuf, &temp);
    }
    for (i=root+1; i<size; i++) {
      MPI_Recv( buffer, count, datatype, i, MPIR_USER_REDUCE_TAG, comm, &status );
	  temp = count;
      (*op)(buffer, recvbuf, &temp);
    }
  }
  else 
    MPI_Send( sendbuf, count, datatype, root, MPIR_USER_REDUCE_TAG, comm );

  return (errno);
}


