/*
 *  $Id: make_cart.c,v 1.5 1994/01/25 21:58:02 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Make_cart - make a new communicator to which topology information
                has been attached

algorithm:
We ignore the periods and reorder info currently.

@*/
/*ARGSUSED*/
int MPI_Make_cart ( comm_old, ndims, dims, periods, reorder, comm_cart )
MPI_Comm  comm_old;
int       ndims;     
int      *dims;     
int      *periods;
int       reorder;
MPI_Comm *comm_cart;
{
  int range[3];
  MPI_Group group_old, group;
  int i, rank, num_ranks = 1;
  int errno = MPI_SUCCESS;

  /* Check validity of arguments */
  if (((comm_old  == (MPI_Comm)0 )  && (errno = MPI_ERR_COMM_NULL)) ||
      ((comm_cart == (MPI_Comm *)0) && (errno = MPI_ERR_NULL))      ||
      ((ndims     <  1)             && (errno = MPI_ERR_DIMS))      ||
      ((dims      == (int *)0)      && (errno = MPI_ERR_DIMS))      ||
      ((periods   == (int *)0)      && (errno = MPI_ERR_UNKNOWN))    )
    return (errno);
  
  /* Determine number of ranks in topology and make new comm */
  for ( i=0; i<ndims; i++ )
    num_ranks    *= (dims[i]>0)?dims[i]:-dims[i];
  if ( num_ranks < 1 ) {
    (*comm_cart)  = (MPI_Comm)0;
    return (errno);
  }
  range[0] = 0; range[1] = num_ranks - 1; range[2] = 1;
  MPI_Comm_group ( comm_old, &group_old );
  MPI_Group_range_incl ( group_old, 1, range, &group );
  MPI_Comm_make  ( comm_old, group, comm_cart );

  /* Store topology information in new communicator */
  if ( (*comm_cart) != (MPI_Comm)0 ) {
    (*comm_cart)->topology.type                  = MPI_CART;
    (*comm_cart)->topology.tag.cart.nnodes       = num_ranks;
    (*comm_cart)->topology.tag.cart.ndims        = ndims;
    (*comm_cart)->topology.tag.cart.dims         = 
      (int *)MPIR_MALLOC( sizeof(int) * 3 * ndims );
    (*comm_cart)->topology.tag.cart.periods      = 
      (*comm_cart)->topology.tag.cart.dims + ndims * 3;
    (*comm_cart)->topology.tag.cart.position     = 
      (*comm_cart)->topology.tag.cart.periods + ndims * 3;
    for ( i=0; i<ndims; i++ ) {
      (*comm_cart)->topology.tag.cart.dims[i]    = dims[i];
      (*comm_cart)->topology.tag.cart.periods[i] = periods[i];
    }

    /* Compute my position */
    MPI_Comm_rank ( (*comm_cart), &rank );
    for ( i=0; i < ndims; i++ ) {
      num_ranks = num_ranks / dims[i];
      (*comm_cart)->topology.tag.cart.position[i]  = rank / num_ranks;
      rank   = rank % num_ranks;
    }
  }
  return (errno);
}
